{
  This unit interfaces SQLite3 libs.
  Copyright (C) 2005 by Eike Lange
  Author: Eike Lange <eike.lange@uni-essen.de>  

  Documentation about SQLite3 is available at http://www.sqlite.org/
  
  This unit uses the same license, as the SQLite3 library, as long as
  writing free software in the FSF sense of the word is allowed.
  The original license is more a "do whatever you want" license,
  and instead of a license is a poem.

  Missing:
  ========
  SQLite3Bind: SQLITE_STATIC and SQLITE_TRANSIENT unimplemnted 
  SQLite3Free, frees memory we don't use...
}

{$gnu-pascal,I-}
{$if __GPC_RELEASE__ < 20050331}
{$error This unit requires GPC release 20050331 or newer.}
{$endif}

unit SQLite3;

interface

uses GPC;

type
  { These datatyes are opaque }
  TSQLite3Context = Pointer;
  TSQLite3Statement = Pointer;
  TSQLite3 = Pointer;
  TSQLite3Value = Pointer;

  PStringSchema = ^TStringSchema;
  TStringSchema (Num: Integer) = array [1 .. Num] of TString;
  
  { for SQLite3Exec }
  TSQLite3Callback = ^function (Arg: Pointer; NumCol: Integer; ColVals, ColNames: PStringSchema): Integer;
  
  { for SQLite3BusyHandler }
  TBusyHandlerCallback = ^function (a: Pointer; b: Integer): Integer;
  
  { for SQLite3Bind[Blob, Text, Text16] functions }
  TBindBlobDestroyProc = ^procedure (x: Pointer);
  
  { for SQLite3Result[Blob, text*] functions }
  TResultBlobProc = ^procedure (a: Pointer);
  
  { for SQLite3SetAuthorizer.
    The callback should return SQLITE_OK if access is allowed, SQLITE_DENY if
    the entire SQL statement should be aborted with an error and SQLITE_IGNORE if
    the column should be treated as a NULL value. }
  TXAuthFunc = ^function (a: Pointer; b: Integer; const c, d, e, f: TString): Integer;
  
  { for SQLite3Trace }
  TXTraceProc =  ^procedure (a: Pointer; const b: TString);
  
  { for SQLite3CollationNeeded }
  TCollationNeededProc = ^procedure (a, b: Pointer; eTextRep: Integer; const s: TString);
  
  { for SQLite3CollationNeeded16 }
  TCollationNeededProc16 = ^procedure (a, b: Pointer; eTextRep: Integer; const s: Pointer);
  
  { for SQLite3CreateCollation / 16 }
  TXCompareFunc = ^function (a: Pointer; b: Integer; const c: Pointer; d: Integer; const e: Pointer): Integer;

  { for cSQLite3CreateFunction* }
  PSQLite3ValueArray = ^TSQLite3ValueArray;
  TSQLite3ValueArray = array [0 .. MaxVarSize div SizeOf (TSQLite3Value) - 1] of TSQLite3Value;
  TXFuncProc = 	^procedure (Ctx: TSQLite3Context; a: Integer; ValueArray: PSQLite3ValueArray);
  TXStepProc = 	^procedure (Ctx: TSQLite3Context; a: Integer; ValueArray: PSQLite3ValueArray);
  TXFinalProc = ^procedure (Ctx: TSQLite3Context);

var
  {@section result codes, see http://www.sqlite.org/capi3ref.html#result-codes }
  { Successful result }
  SQLITE_OK: CInteger; external name '_c_CSQLITE_OK'; attribute (const);
  
  { SQL error or missing database }
  SQLITE_ERROR: CInteger; external name '_c_CSQLITE_ERROR'; attribute (const);
  
  { An internal logic error in SQLite }
  SQLITE_INTERNAL: CInteger; external name '_c_CSQLITE_INTERNAL'; attribute (const);
  
  { Access permission denied }
  SQLITE_PERM: CInteger; external name '_c_CSQLITE_PERM'; attribute (const);
  
  { Callback routine requested an abort }
  SQLITE_ABORT: CInteger; external name '_c_CSQLITE_ABORT'; attribute (const);
  
  { The database file is locked }
  SQLITE_BUSY: CInteger; external name '_c_CSQLITE_BUSY'; attribute (const);
  
  { A table in the database is locked }
  SQLITE_LOCKED: CInteger; external name '_c_CSQLITE_LOCKED'; attribute (const);
  
  { A malloc() failed }
  SQLITE_NOMEM: CInteger; external name '_c_CSQLITE_NOMEM'; attribute (const);
  
  { Attempt to write a readonly database }
  SQLITE_READONLY: CInteger; external name '_c_CSQLITE_READONLY'; attribute (const);
  
  { Operation terminated by sqlite_interrupt() }
  SQLITE_INTERRUPT: CInteger; external name '_c_CSQLITE_INTERRUPT'; attribute (const);
  
  { Some kind of disk I/O error occurred }
  SQLITE_IOERR: CInteger; external name '_c_CSQLITE_IOERR'; attribute (const);

  { The database disk image is malformed }
  SQLITE_CORRUPT: CInteger; external name '_c_CSQLITE_CORRUPT'; attribute (const);

  { (Internal Only) Table or record not found }
  SQLITE_NOTFOUND: CInteger; external name '_c_CSQLITE_NOTFOUND'; attribute (const);

  { Insertion failed because database is full }
  SQLITE_FULL: CInteger; external name '_c_CSQLITE_FULL'; attribute (const);

  { Unable to open the database file }
  SQLITE_CANTOPEN: CInteger; external name '_c_CSQLITE_CANTOPEN'; attribute (const);

  { Database lock protocol error }
  SQLITE_PROTOCOL: CInteger; external name '_c_CSQLITE_PROTOCOL'; attribute (const);

  { (Internal Only) Database table is empty }
  SQLITE_EMPTY: CInteger; external name '_c_CSQLITE_EMPTY'; attribute (const);

  { The database schema changed }
  SQLITE_SCHEMA: CInteger; external name '_c_CSQLITE_SCHEMA'; attribute (const);

  { Too much data for one row of a table }
  SQLITE_TOOBIG: CInteger; external name '_c_CSQLITE_TOOBIG'; attribute (const);

  { Abort due to constraint violation }
  SQLITE_CONSTRAINT: CInteger; external name '_c_CSQLITE_CONSTRAINT'; attribute (const);

  { Data type mismatch }
  SQLITE_MISMATCH: CInteger; external name '_c_CSQLITE_MISMATCH'; attribute (const);

  { Library used incorrectly }
  SQLITE_MISUSE: CInteger; external name '_c_CSQLITE_MISUSE'; attribute (const);

  { Uses OS features not supported on host }
  SQLITE_NOLFS: CInteger; external name '_c_CSQLITE_NOLFS'; attribute (const);

  { Authorization denied }
  SQLITE_AUTH: CInteger; external name '_c_CSQLITE_AUTH'; attribute (const);

  { sqlite_step() has another row ready }
  SQLITE_ROW: CInteger; external name '_c_CSQLITE_ROW'; attribute (const);

  { sqlite_step() has finished executing }
  SQLITE_DONE: CInteger; external name '_c_CSQLITE_DONE'; attribute (const);

  { For SQLite3ColumnType }
  SQLITE_INTEGER: CInteger; external name '_c_CSQLITE_INTEGER'; attribute (const);
  SQLITE_FLOAT: CInteger; external name '_c_CSQLITE_FLOAT'; attribute (const);
  SQLITE_TEXT: CInteger; external name '_c_CSQLITE_TEXT'; attribute (const);
  SQLITE_BLOB: CInteger; external name '_c_CSQLITE_BLOB'; attribute (const);
  SQLITE_NULL: CInteger; external name '_c_CSQLITE_NULL'; attribute (const);

  { For SQLite3CreateCollation }
  SQLITE_UTF8: CInteger; external name '_c_CSQLITE_UTF8'; attribute (const);
  SQLITE_UTF16LE: CInteger; external name '_c_CSQLITE_UTF16LE'; attribute (const);
  SQLITE_UTF16BE: CInteger; external name '_c_CSQLITE_UTF16BE'; attribute (const);
  SQLITE_UTF16: CInteger; external name '_c_CSQLITE_UTF16'; attribute (const);
  SQLITE_ANY: CInteger; external name '_c_CSQLITE_ANY'; attribute (const);

  { for TXAuthFunc }
  SQLITE_COPY: CInteger; external name '_c_CSQLITE_COPY'; attribute (const);
  SQLITE_CREATE_INDEX: CInteger; external name '_c_CSQLITE_CREATE_INDEX'; attribute (const);
  SQLITE_CREATE_TABLE: CInteger; external name '_c_CSQLITE_CREATE_TABLE'; attribute (const);
  SQLITE_CREATE_TEMP_INDEX: CInteger; external name '_c_CSQLITE_CREATE_TEMP_INDEX'; attribute (const);
  SQLITE_CREATE_TEMP_TABLE: CInteger; external name '_c_CSQLITE_CREATE_TEMP_TABLE'; attribute (const);
  SQLITE_CREATE_TEMP_TRIGGER: CInteger; external name '_c_CSQLITE_CREATE_TEMP_TRIGGER'; attribute (const);
  SQLITE_CREATE_TEMP_VIEW: CInteger; external name '_c_CSQLITE_CREATE_TEMP_VIEW'; attribute (const);
  SQLITE_CREATE_TRIGGER: CInteger; external name '_c_CSQLITE_CREATE_TRIGGER'; attribute (const);
  SQLITE_CREATE_VIEW: CInteger; external name '_c_CSQLITE_CREATE_VIEW'; attribute (const);
  SQLITE_DELETE: CInteger; external name '_c_CSQLITE_DELETE'; attribute (const);
  SQLITE_DROP_INDEX: CInteger; external name '_c_CSQLITE_DROP_INDEX'; attribute (const);
  SQLITE_DROP_TABLE: CInteger; external name '_c_CSQLITE_DROP_TABLE'; attribute (const);
  SQLITE_DROP_TEMP_INDEX: CInteger; external name '_c_CSQLITE_DROP_TEMP_INDEX'; attribute (const);
  SQLITE_DROP_TEMP_TABLE: CInteger; external name '_c_CSQLITE_DROP_TEMP_TABLE'; attribute (const);
  SQLITE_DROP_TEMP_TRIGGER: CInteger; external name '_c_CSQLITE_DROP_TEMP_TRIGGER'; attribute (const);
  SQLITE_DROP_TEMP_VIEW: CInteger; external name '_c_CSQLITE_DROP_TEMP_VIEW'; attribute (const);
  SQLITE_DROP_TRIGGER: CInteger; external name '_c_CSQLITE_DROP_TRIGGER'; attribute (const);
  SQLITE_DROP_VIEW: CInteger; external name '_c_CSQLITE_DROP_VIEW'; attribute (const);
  SQLITE_INSERT: CInteger; external name '_c_CSQLITE_INSERT'; attribute (const);
  SQLITE_PRAGMA: CInteger; external name '_c_CSQLITE_PRAGMA'; attribute (const);
  SQLITE_READ: CInteger; external name '_c_CSQLITE_READ'; attribute (const);
  SQLITE_SELECT: CInteger; external name '_c_CSQLITE_SELECT'; attribute (const);
  SQLITE_TRANSACTION: CInteger; external name '_c_CSQLITE_TRANSACTION'; attribute (const);
  SQLITE_UPDATE: CInteger; external name '_c_CSQLITE_UPDATE'; attribute (const);
  SQLITE_ATTACH: CInteger; external name '_c_CSQLITE_ATTACH'; attribute (const);
  SQLITE_DETACH: CInteger; external name '_c_CSQLITE_DETACH'; attribute (const);
  SQLITE_DENY: CInteger; external name '_c_CSQLITE_DENY'; attribute (const);
  SQLITE_IGNORE: CInteger; external name '_c_CSQLITE_IGNORE'; attribute (const);

{ Stors state in Ctx }
function SQLite3AggregateContext (Ctx: TSQLite3Context; Count: Integer): Pointer; external name '_c_Csqlite3_aggregate_context';

{ Counts number of calls to xStep function }
function SQLite3AggregateCount (Ctx: TSQLite3Context): Integer; external name '_c_Csqlite3_aggregate_count';

{ Binds value to prepared statements. i is parameter to bind (starts by 1). 3rd parameter is value to bind. }
function SQLite3BindBlob (Stmnt: TSQLite3Statement; i: Integer; const p: Pointer; Len: Integer; Proc: TBindBlobDestroyProc): Integer;external name '_c_Csqlite3_bind_blob';
function SQLite3BindDouble (Stmnt: TSQLite3Statement; i: Integer; ValueToBind: Double): Integer; external name '_c_Csqlite3_bind_double';
function SQLite3BindInt (Stmnt: TSQLite3Statement; i: Integer; ValueToBind: Integer): Integer; external name '_c_Csqlite3_bind_int';
function SQLite3BindInt64 (Stmnt: TSQLite3Statement; i: Integer; ValueToBind: LongInt): Integer; external name '_c_Csqlite3_bind_int64';
function SQLite3BindNull (Stmnt: TSQLite3Statement; i: Integer): Integer; external name '_c_Csqlite3_bind_null';

{ Return the number of parameters in the precompiled statement given as the argument. }
function SQLite3BindParameterCount (Stmnt: TSQLite3Statement): Integer; external name '_c_Csqlite3_bind_parameter_count';

{ Return the index of the parameter with the given UTF-8 name. The name must match exactly. Else return 0}
function SQLite3BindParameterIndex (Stmnt: TSQLite3Statement; const zName: TString): Integer; external name '_c_Csqlite3_bind_parameter_index';

{ Return the name of the n-th parameter in the precompiled statement. }
function SQLite3BindParameterName (Stmnt: TSQLite3Statement; i: Integer): TString;

{ Bind parameter i in a prepared statement Stmnt to String TxtUtf8.
  If the parameter must be deleted, Proc can do this for you. }
function SQLite3BindText (Stmnt: TSQLite3Statement; i: Integer; TxtUtf8: TString; Len: Integer; Proc: TBindBlobDestroyProc): Integer;

{ same as above for UTF-16 text }
function SQLite3BindText16 (Stmnt: TSQLite3Statement; i: Integer; TxtUtf16: Pointer; Len: Integer; Proc: TBindBlobDestroyProc): CInteger; external name '_c_Csqlite3_bind_text16';

{ Set a busy handler, which is invoked, when attempts to access a locked table is made }
function SQLite3BusyHandler (SQL: TSQLite3; Callback: TBusyHandlerCallback; Something: Pointer): Integer;

{ busy handler should sleep for a while.. }
function SQLite3BusyTimeout (SQL: TSQLite3; MilliSeconds: Integer): Integer; external name '_c_Csqlite3_busy_timeout';

{ returns number of changed rows by most recent INSERT, UPDATE, DELETE statement}
function SQLite3Changes (SQL: TSQLite3): Integer; external name '_c_Csqlite3_changes';

{ Closes a Database opened by SQLite3Open or SQLite3Open16. SQLITE_OK is returned, if everything works fine,
  and SQLITE_BUSY or SQLITE_ERROR in case of an error }
function SQLite3Close (SQL: TSQLite3): Integer; external name '_c_Csqlite3_close'; attribute (ignorable);

{ A single callback function may be registered with the database handle to be called whenever an
  undefined collation sequence is required.}
function SQLite3CollationNeeded (a: Pointer; b: Pointer; f: TCollationNeededProc): Integer;
function SQLite3CollationNeeded16 (a: Pointer; b: Pointer; f: TCollationNeededProc16): Integer;

{ Returns data in column iCol for BLOB columns }
function SQLite3ColumnBlob (Stmnt: TSQLite3Statement; iCol: Integer): Pointer; external name '_c_Csqlite3_column_blob';

{ Returns number of bytes in BLOB or TEXT columns }
function SQLite3ColumnBytes (Stmnt: TSQLite3Statement; iCol: Integer): Integer; external name '_c_Csqlite3_column_bytes';

{ Returns number of bytes in BLOB or TEXT16 columns }
function SQLite3ColumnBytes16 (Stmnt: TSQLite3Statement; iCol: Integer): Integer; external name '_c_Csqlite3_column_bytes16';

{ Return the number of columns in the result set returned by the prepared SQL statement. }
function SQLite3ColumnCount (Stmnt: TSQLite3Statement): Integer; external name '_c_Csqlite3_column_count';

{ Return type of column or null } 
function SQLite3ColumnDecltype (Stmnt: TSQLite3Statement; i: Integer): TString;
function SQLite3ColumnDecltype16 (Stmnt: TSQLite3Statement; i: Integer): Pointer; external name '_c_Csqlite3_column_decltype16';

{ Returns data in column iCol for FLOAT columns }
function SQLite3ColumnDouble (Stmnt: TSQLite3Statement; iCol: Integer): Double; external name '_c_Csqlite3_column_double';

{ Returns data in column iCol for INTEGER columns }
function SQLite3ColumnInt (Stmnt: TSQLite3Statement; iCol: Integer): Integer; external name '_c_Csqlite3_column_int';

{ Returns data in column iCol for INTEGER columns }
function SQLite3ColumnInt64 (Stmnt: TSQLite3Statement; iCol: Integer): LongInt; external name '_c_Csqlite3_column_int64';

{ This function returns the column heading for the Nth column of that statement }
function SQLite3ColumnName (Stmnt: TSQLite3Statement; iCol: Integer): TString;
function SQLite3ColumnName16 (Stmnt: TSQLite3Statement; iCol: Integer): Pointer; external name '_c_Csqlite3_column_name16';

{ returns data for a text column }
function SQLite3ColumnText (Stmnt: TSQLite3Statement; iCol: Integer): TString;

{ same as above for UTF-16 }
function SQLite3ColumnText16 (Stmnt: TSQLite3Statement; iCol: Integer): Pointer; external name '_c_Csqlite3_column_text16';

{ returns SQLITE_INTEGER...SQLITE_NULL }
function SQLite3ColumnType (Stmnt: TSQLite3Statement; iCol: Integer): Integer; external name '_c_Csqlite3_column_type';

{ TRUE, if s or sUtf16 is one or more SQL statements }
function SQLite3Complete (s: TString): Boolean; 
function SQLite3Complete16 (sUtf16: Pointer): Boolean;

{ These two functions are used to add new collation sequences to the
  TSQLite3 handle specified as the first argument. }
function SQLite3CreateCollation (SQL: TSQLite3; const zName: TString; eTextRep: Integer; p1: Pointer; Func: TXCompareFunc): Integer;
function SQLite3CreateCollation16 (SQL: TSQLite3; const zName: TString; eTextRep: Integer; p1: Pointer; Func: TXCompareFunc): Integer;

{ These two functions are used to add SQL functions or aggregates }
function SQLite3CreateFunction (p1: Pointer; const FuncName: TString; nArg, eTxtRep: Integer; pUserData: Pointer; f1: TXFuncProc; f2: TXStepProc; f3: TXFinalProc): Integer; external name '_c_Csqlite3_create_function';
function SQLite3CreateFunction16 (p1: Pointer; const FuncName: Pointer; nArg, eTxtRep: Integer; pUserData: Pointer; f1: TXFuncProc; f2: TXStepProc; f3: TXFinalProc): Integer; external name '_c_Csqlite3_create_function16';

{ Return the number of values in the current row of the result set. }
function SQLite3DataCount (Stmnt: TSQLite3Statement): Integer; external name '_c_Csqlite3_data_count';

{ Return the error code for the most recent failed SQLite3-Call }
function SQLite3Errcode (Db: TSQLite3): Integer; external name '_c_Csqlite3_errcode';

{ Error string describing the error, UTF-8 }
function SQLite3ErrMsg (Db: TSQLite3): TString;

{ Error string describing the error, UTF-16 }
function SQLite3ErrMsg16 (Db: TSQLite3): Pointer; external name '_c_Csqlite3_errmsg16';

{ Executes one or more queries. Func should normally return 0 and cn be NULL. Arg is the first 
  argument of Func. } 
function SQLite3Exec (Db: TSQLite3; const SQL: TString; Func: TSQLite3Callback; Arg: Pointer; var ErrMsg: TString): Integer; attribute (ignorable);

{ Delete a prepared SQL statement. If the statement was executed successfully, or not executed at all,
  then SQLITE_OK is returned, else error code is returned. All prepared statements must finalized
  before SQLite3Close is called or else the close will fail with a return code of SQLITE_BUSY. }
function SQLite3Finalize (Stmnt: TSQLite3Statement): Integer; external name '_c_Csqlite3_finalize'; attribute (ignorable);

{ Disposes the memory allocated by `SQLite3GetTable'. In fact, it disposes every PStringSchema ;-) }
procedure SQLite3FreeTable (TableData: PStringSchema);

{ TableData^[i] holds all Table data, including the header. NumRows holds only the data rows,
  NumCols is correct. }
function SQLite3GetTable (Db: TSQLite3; const SQLStmnt: TString; var TableData: PStringSchema;
  var NumRows, NumCols: Integer; var ErrMsg: TString): Integer; attribute (ignorable);

{ abort any database operations }
procedure SQLite3Interrupt (Db: TSQLite3); external name '_c_Csqlite3_interrupt';

{ returns most recent integer key of INSERT operation }
function SQLite3LastInsertRowId (Db: TSQLite3): LongInt; external name '_c_Csqlite3_last_insert_rowid';

{ Version number of the library (e.g. 3.0.8)}
function SQLite3Libversion: TString;

{ Opens or creates a database. The handle to the database is returned in "Db", the
  function returns "SQLITE_OK" if everything works fine. }
function SQLite3Open (const Fn: TString; var Db: TSQLite3): Integer; external name '_c_Csqlite3_open'; attribute (ignorable); 
function SQLite3Open16 (const Fn: Pointer; var Db: TSQLite3): Integer; external name '_c_Csqlite3_open16'; attribute (ignorable);

{ Prepare a statement (SQL query). zSql is the query, nBytes is the length of the string. The
  statement itself is given in Stmnt. pzTail is just the portion of zSql, which remains uncompiled.
  On success, SQLITE_OK is returned, else an error code }
function SQLite3Prepare (Db: TSQLite3; const zSql: TString; nBytes: Integer; var Stmnt: TSQLite3Statement; var pzTail: TString): Integer;
function SQLite3Prepare16 (Db: TSQLite3; const zSql: Pointer; nBytes: Integer; var Stmnt: TSQLite3Statement; var pzTail: Pointer): Integer; external name '_c_Csqlite3_prepare16';

{ reset a prepared SQL statement }
function SQLite3Reset (Stmnt: TSQLite3Statement): Integer; external name '_c_Csqlite3_reset';

{ User-defined functions invoke these routines in order to set their return value. }
procedure SQLite3ResultBlob (Ctx: TSQLite3Context; const p: Pointer; n: Integer; Proc: TResultBlobProc); external name '_c_Csqlite3_result_blob';
procedure SQLite3ResultDouble (Ctx: TSQLite3Context; d: Double); external name '_c_Csqlite3_result_double';
procedure SQLite3ResultError (Ctx: TSQLite3Context; const s: TString; n: Integer); external name '_c_Csqlite3_result_error';
procedure SQLite3ResultError16 (Ctx: TSQLite3Context; const p: Pointer; n: Integer); external name '_c_Csqlite3_result_error16';
procedure SQLite3ResultInt (Ctx: TSQLite3Context; n: Integer); external name '_c_Csqlite3_result_int';
procedure SQLite3ResultInt64 (Ctx: TSQLite3Context; r: LongInt); external name '_c_Csqlite3_result_int64';
procedure SQLite3ResultNull (Ctx: TSQLite3Context); external name '_c_Csqlite3_result_null';
procedure SQLite3ResultText (Ctx: TSQLite3Context; const s: TString; n: Integer; Proc: TResultBlobProc); external name '_c_Csqlite3_result_text';
procedure SQLite3ResultText16 (Ctx: TSQLite3Context; const s: Pointer; n: Integer; Proc: TResultBlobProc); external name '_c_Csqlite3_result_text16';
procedure SQLite3ResultText16Be (Ctx: TSQLite3Context; const s: Pointer; n: Integer; Proc: TResultBlobProc); external name '_c_Csqlite3_result_text16be';
procedure SQLite3ResultText16Le (Ctx: TSQLite3Context; const s: Pointer; n: Integer; Proc: TResultBlobProc); external name '_c_Csqlite3_result_text16le';

{ return an exact copy of one of the arguments to the function. }
procedure SQLite3ResultValue (Ctx: TSQLite3Context; Valuee: TSQLite3Value); external name '_c_Csqlite3_result_value';

{ This routine registers a callback with the SQLite library. The callback is invoked
  (at compile-time, not at run-time) for each attempt to access a column of a table
  in the database. }
function SQLite3SetAuthorizer (SQL: TSQLite3; Func: TXAuthFunc; UserData: Pointer): Integer;

{ After an SQL query has been prepared with a call to either SQLite3Prepare or SQLite3Prepare16,
  then this function must be called one or more times to execute the statement.
  The return value will be either SQLITE_BUSY, SQLITE_DONE, SQLITE_ROW, SQLITE_ERROR, or
  SQLITE_MISUSE.}
function SQLite3Step (Stmnt: TSQLite3Statement): Integer; external name '_c_Csqlite3_step';

{ This function returns the total number of database rows that have be modified,
  inserted, or deleted since the database connection was created using
  SQLite3Open. }
function SQLite3TotalChanges (SQL: TSQLite3): Integer; external name '_c_Csqlite3_total_changes';

{ Register a function that is called each time an SQL statement is evaluated } 
function SQLite3Trace (SQL: TSQLite3; Func: TXTraceProc; a: Pointer): Pointer;

{ documentation unclear }
function SQLite3UserData (Ctx: TSQLite3Context): Pointer; external name '_c_Csqlite3_user_data';

{ This group of routines returns information about arguments to a user-defined function. }
function SQLite3ValueBlob (Valuee: TSQLite3Value): Pointer; external name '_c_Csqlite3_value_blob';
function SQLite3ValueBytes (Valuee: TSQLite3Value): Integer; external name '_c_Csqlite3_value_bytes';
function SQLite3ValueBytes16 (Valuee: TSQLite3Value): Integer; external name '_c_Csqlite3_value_bytes16';
function SQLite3ValueDouble (Valuee: TSQLite3Value): Double; external name '_c_Csqlite3_value_double';
function SQLite3ValueInt (Valuee: TSQLite3Value): Integer; external name '_c_Csqlite3_value_int';
function SQLite3ValueInt64 (Valuee: TSQLite3Value): LongInt; external name '_c_Csqlite3_value_int64';
function SQLite3ValueText (Valuee: TSQLite3Value): TString; external name '_c_Csqlite3_value_text';
function SQLite3ValueText16 (Valuee: TSQLite3Value): Pointer; external name '_c_Csqlite3_value_text16';
function SQLite3ValueText16Be (Valuee: TSQLite3Value): Pointer; external name '_c_Csqlite3_value_text16be';
function SQLite3ValueText16Le (Valuee: TSQLite3Value): Pointer; external name '_c_Csqlite3_value_text16le';
function SQLite3ValueType (Valuee: TSQLite3Value): Integer; external name '_c_Csqlite3_value_type';

implementation

{$L sqlite3c.c,sqlite3}

type
  { for cSQLite3BusyHandler }
  PBusyCallback = ^function (x: Pointer; y: CInteger): CInteger;

  { for cSQLite3CollatinNeeded* }
  PCollationNeededProc = ^procedure (a: Pointer; b: Pointer; eTextRep: CInteger; const s: CString);
  PCollationNeededProc16 = ^procedure (a: Pointer; b: Pointer; eTextRep: CInteger; const s: Pointer);
  
  PBusyHandlerFunc =   ^function (a: Pointer; b: CInteger): CInteger;
  PXCompareFunc =      ^function (a: Pointer; b: CInteger; const c: Pointer; d: CInteger; const e: Pointer): CInteger;

  { for cSQLite3GetTable, SQLite3Exec }
  PSQLite3CStringArray = ^SQLite3CStringArray;
  SQLite3CStringArray = array [0 .. MaxVarSize div SizeOf (CString) - 1] of CString;

  { for SQLite3Exec. Arg is from SQLite3Exec, NumCol is the number of columns.
    ColVals is a C-array of Column strings and ColNames is an array [0..NumCols]
    of column names }
  PCSQLite3CallbackFunc = ^function (Arg: Pointer; NumCol: CInteger; ColVals, ColNames: PSQLite3CStringArray): CInteger;
  
  PXAuthFunc =         ^function (a: Pointer; b: CInteger; const c,d,e,f: CString): CInteger;
  PXTraceProc =        ^procedure (a: Pointer; const b: CString);

function cSQLite3BindParameterName (Stmnt: TSQLite3Statement; i: CInteger): {const} CString; external name '_c_Csqlite3_bind_parameter_name';
function SQLite3BindParameterName (Stmnt: TSQLite3Statement; i: Integer): TString;
var Ret: CString;
begin
  Ret := cSQLite3BindParameterName (Stmnt, i);
  SQLite3BindParameterName := CString2String (Ret)
end;

function cSQLite3BindText (Stmnt: TSQLite3Statement; i: CInteger; TxtUtf8: CString; Len: CInteger; Proc: TBindBlobDestroyProc): CInteger; external name '_c_Csqlite3_bind_text';
function SQLite3BindText (Stmnt: TSQLite3Statement; i: Integer; TxtUtf8: TString; Len: Integer; Proc: TBindBlobDestroyProc): Integer;
begin
  SQLite3BindText := cSQLite3BindText (Stmnt, i, CString2String (TxtUtf8), Len, Proc)
end;

function cSQLite3BusyHandler (SQL: TSQLite3; Callback: PBusyCallback; Something: Pointer): CInteger; external name '_c_Csqlite3_busy_handler';
function SQLite3BusyHandler (SQL: TSQLite3; Callback: TBusyHandlerCallback; Something: Pointer): Integer;
var
  Ret: CInteger;
  function CCallback (x: Pointer; y: CInteger): CInteger;
  begin
    CCallback := Callback^ (x, y)
  end;
begin
  if Callback = nil then
    Ret := cSQLite3BusyHandler (SQL, nil, Something)
  else
    Ret := cSQLite3BusyHandler (SQL, @CCallback, Something);
  SQLite3BusyHandler := Ret
end;

function cSQLite3CollationNeeded (a: Pointer; b: Pointer; f: PCollationNeededProc): CInteger; external name '_c_Csqlite3_collation_needed';
function SQLite3CollationNeeded (a: Pointer; b: Pointer; f: TCollationNeededProc): Integer;
var Ret: CInteger;
  procedure Callback (a: Pointer; b: Pointer; eTextRep: CInteger; const s: CString);
  begin
    f^ (a, b, eTextRep, CString2String (s))
  end;
begin
  if f <> nil then
    Ret := cSQLite3CollationNeeded (a, b, @Callback)
  else
    Ret := cSQLite3CollationNeeded (a, b, nil);
  SQLite3CollationNeeded := Ret
end;

function cSQLite3CollationNeeded16 (a: Pointer; b: Pointer; f: PCollationNeededProc16): CInteger; external name '_c_Csqlite3_collation_needed16';
function SQLite3CollationNeeded16 (a: Pointer; b: Pointer; f: TCollationNeededProc16): Integer;
var Ret: CInteger;
  procedure Callback (a: Pointer; b: Pointer; eTextRep: CInteger; const s: Pointer);
  begin
    f^ (a, b, eTextRep, s)
  end;
begin
  if f <> nil then
    Ret := cSQLite3CollationNeeded16 (a, b, @Callback)
  else
    Ret := cSQLite3CollationNeeded16 (a, b, nil);
  SQLite3CollationNeeded16 := Ret
end;

function cSQLite3ColumnDecltype (Stmnt: TSQLite3Statement; i: CInteger): {const} CString; external name '_c_Csqlite3_column_decltype';
function SQLite3ColumnDecltype (Stmnt: TSQLite3Statement; i: Integer): TString;
begin
  SQLite3ColumnDecltype := CString2String (cSQLite3ColumnDecltype (Stmnt, i))
end;

function cSQLite3ColumnName (Stmnt: TSQLite3Statement; iCol: CInteger): {const} CString; external name '_c_Csqlite3_column_name';
function SQLite3ColumnName (Stmnt: TSQLite3Statement; iCol: Integer): TString;
begin
  SQLite3ColumnName := CString2String (cSQLite3ColumnName (Stmnt, iCol))
end;

function cSQLite3ColumnText (Stmnt: TSQLite3Statement; iCol: CInteger): {const} CString; external name '_c_Csqlite3_column_text';
function SQLite3ColumnText (Stmnt: TSQLite3Statement; iCol: Integer): TString;
begin
  SQLite3ColumnText := CString2String (cSQLite3ColumnText (Stmnt, iCol))
end;

function cSQLite3Complete (s: CString): CInteger; external name '_c_Csqlite3_complete';
function SQLite3Complete (s: TString): Boolean;
begin
  SQLite3Complete := cSQLite3Complete (s) = 1
end;

function cSQLite3Complete16 (sUtf16: Pointer): CInteger; external name '_c_Csqlite3_complete16';
function SQLite3Complete16 (sUtf16: Pointer): Boolean;
begin
  SQLite3Complete16 := cSQLite3Complete16 (sUtf16) = 1
end;

function cSQLite3CreateCollation (SQL: TSQLite3; const zName: CString; eTextRep: CInteger; p1: Pointer; Func: PXCompareFunc): CInteger; external name '_c_Csqlite3_create_collation';
function SQLite3CreateCollation (SQL: TSQLite3; const zName: TString; eTextRep: Integer; p1: Pointer; Func: TXCompareFunc): Integer;
var Ret: CInteger;
  function Callback (a: Pointer; b: CInteger; const c: Pointer; d: CInteger; const e: Pointer): CInteger;
  begin
    Callback := Func^ (a, b, c, d, e)
  end;
begin
  if Func <> nil then
    Ret := cSQLite3CreateCollation (SQL, zName, eTextRep, p1, @Callback)
  else
    Ret := cSQLite3CreateCollation (SQL, zName, eTextRep, p1, nil);
  SQLite3CreateCollation := Ret
end;

function cSQLite3CreateCollation16 (SQL: TSQLite3; const zName: CString; eTextRep: CInteger; p1: Pointer; Func: PXCompareFunc): CInteger; external name '_c_Csqlite3_create_collation16';
function SQLite3CreateCollation16 (SQL: TSQLite3; const zName: TString; eTextRep: Integer; p1: Pointer; Func: TXCompareFunc): Integer;
var Ret: CInteger;
  function Callback (a: Pointer; b: CInteger; const c: Pointer; d: CInteger; const e: Pointer): CInteger;
  begin
    Callback := Func^ (a, b, c, d, e)
  end;
begin
  if Func <> nil then
    Ret := cSQLite3CreateCollation16 (SQL, zName, eTextRep, p1, @Callback)
  else
    Ret := cSQLite3CreateCollation16 (SQL, zName, eTextRep, p1, nil);
  SQLite3CreateCollation16 := Ret
end;

function cSQLite3ErrMsg (Db: TSQLite3): {const} CString; external name '_c_Csqlite3_errmsg';
function SQLite3ErrMsg (Db: TSQLite3): TString;
var Ret: CString;
begin
  Ret := cSQLite3ErrMsg (Db);
  SQLite3ErrMsg := CString2String (Ret)
end;

function cSQLite3Exec (Db: TSQLite3; const SQL: CString; Func: PCSQLite3CallbackFunc; CallbackArg: Pointer; var ErrMsg: CString): CInteger; external name '_c_Csqlite3_exec';
function SQLite3Exec (Db: TSQLite3; const SQL: TString; Func: TSQLite3Callback; Arg: Pointer; var ErrMsg: TString): Integer;
var
  Ret: CInteger;
  Err: CString;
  function Callback (Arg: Pointer; NumCol: CInteger; ColVals, ColNames: PSQLite3CStringArray): CInteger;
  var 
    ColValues, ColHeadings: PStringSchema;
    i: Integer;
  begin
    New (ColValues, NumCol);
    New (ColHeadings, NumCol);
    for i := 0 to NumCol - 1 do
      begin
        ColValues^[i + 1] := CString2String (ColVals^[i]);
        ColHeadings^[i + 1] := CString2String (ColNames^[i]);
      end;
    Callback := Func^ (Arg, NumCol, ColValues, ColHeadings)
  end;
begin
  if Func = nil then
    Ret := cSQLite3Exec (Db, SQL, nil, nil, Err)
  else
    Ret := cSQLite3Exec (Db, SQL, @Callback, Arg, Err);
  ErrMsg := CString2String (Err);
  SQLite3Exec := Ret
end;

{ you don't want to use it... }
procedure cSQLite3Free (Z: CString); external name '_c_Csqlite3_free';

procedure cSQLite3FreeTable (ResultP: PSQLite3CStringArray); external name '_c_Csqlite3_free_table';
procedure SQLite3FreeTable (TableData: PStringSchema);
begin
  Dispose (TableData)
end;

function cSQLite3GetTable (Db: TSQLite3; const SQLStmnt: CString; var ResultP: PSQLite3CStringArray; var NRow: CInteger; var NCol: CInteger; var ErrMsg: CString): CInteger; external name '_c_Csqlite3_get_table';
function SQLite3GetTable (Db: TSQLite3; const SQLStmnt: TString; var TableData: PStringSchema; var NumRows, NumCols: Integer; var ErrMsg: TString): Integer;
var
  Ret, nRows, nCols: CInteger;
  CResult: PSQLite3CStringArray;
  Err: CString;
  i: Integer;
begin
  Ret := cSQLite3GetTable (Db, SQLStmnt, CResult, nRows, nCols, Err);
  NumRows := nRows;
  NumCols := nCols;
  New (TableData, (NumRows + 1) * NumCols);
  for i := 0 to ((NumRows + 1) * NumCols) - 1 do
    TableData^[i+1] := CString2String (CResult^[i]);
  cSQLite3FreeTable (CResult);
  ErrMsg := CString2String (Err);
  SQLite3GetTable := Ret
end;

function cSQLite3Libversion: {const} CString; external name '_c_Csqlite3_libversion';
function SQLite3Libversion: TString;
begin
  SQLite3Libversion := CString2String (cSQLite3Libversion)
end;

function cSQLite3Prepare (Db: TSQLite3; const zSql: CString; nBytes: CInteger; var Stmnt: TSQLite3Statement; var pzTail: CString): CInteger; external name '_c_Csqlite3_prepare';
function SQLite3Prepare (Db: TSQLite3; const zSql: TString; nBytes: Integer; var Stmnt: TSQLite3Statement; var pzTail: TString): Integer;
var
  Ret: CInteger;
  cpzTail: CString;
begin
  Ret := cSQLite3Prepare (Db, zSql, nBytes, Stmnt, cpzTail);
  pzTail := CString2String (cpzTail);
  SQLite3Prepare := Ret
end;

function cSQLite3SetAuthorizer (SQL: TSQLite3; Func: PXAuthFunc; UserData: Pointer): CInteger; external name '_c_Csqlite3_set_authorizer';
function SQLite3SetAuthorizer (SQL: TSQLite3; Func: TXAuthFunc; UserData: Pointer): Integer;
var Ret: CInteger;
  function Callback (a: Pointer; b: CInteger; const c,d,e,f: CString): CInteger;
  begin
    Callback := Func^ (a, b, CString2String (c), CString2String (d), CString2String (e), CString2String (f))
  end;
begin
  if Func <> nil then
    Ret := cSQLite3SetAuthorizer (SQL, @Callback, UserData)
  else
    Ret := cSQLite3SetAuthorizer (SQL, nil, UserData);
  SQLite3SetAuthorizer := Ret
end;

function cSQLite3Trace (SQL: TSQLite3; Func: PXTraceProc; a: Pointer): Pointer; external name '_c_Csqlite3_trace';
function SQLite3Trace (SQL: TSQLite3; Func: TXTraceProc; a: Pointer): Pointer;
  procedure Callback (a: Pointer; const b: CString);
  begin
    Func^ (a, CString2String (b))
  end;
begin
  if Func = nil then
    SQLite3Trace:= cSQLite3Trace (SQL, nil, a)
  else
    SQLite3Trace:= cSQLite3Trace (SQL, @Callback, a)
end;



end.
