
/**
  Copyright (C) 2005 Eike Lange
  Author: Eike Lange <eike.lange@uni-essen.de>

  Support routine for sqlite3.pas, a unit for accessing SQLite3

  This file is part of sqlite3.pas

  @@ Version 3.0.8
  @@ Missing:
    char *sqlite3_mprintf(const char*,...)
    char *sqlite3_vmprintf(const char*, va_list);
    void *sqlite3_get_auxdata(sqlite3_context*, int);
    void sqlite3_set_auxdata(sqlite3_context*, int, void*, void (*)(void*));
    extern const char *sqlite3_temp_directory;
**/

#include <sqlite3.h>



/* The return codes */
const int _c_CSQLITE_OK         = SQLITE_OK;
const int _c_CSQLITE_ERROR      = SQLITE_ERROR;
const int _c_CSQLITE_INTERNAL   = SQLITE_INTERNAL;
const int _c_CSQLITE_PERM       = SQLITE_PERM;
const int _c_CSQLITE_ABORT      = SQLITE_ABORT;
const int _c_CSQLITE_BUSY       = SQLITE_BUSY;
const int _c_CSQLITE_LOCKED     = SQLITE_LOCKED;
const int _c_CSQLITE_NOMEM      = SQLITE_NOMEM;
const int _c_CSQLITE_READONLY   = SQLITE_READONLY;
const int _c_CSQLITE_INTERRUPT  = SQLITE_INTERRUPT;
const int _c_CSQLITE_IOERR      = SQLITE_IOERR;
const int _c_CSQLITE_CORRUPT    = SQLITE_CORRUPT;
const int _c_CSQLITE_NOTFOUND   = SQLITE_NOTFOUND;
const int _c_CSQLITE_FULL       = SQLITE_FULL;
const int _c_CSQLITE_CANTOPEN   = SQLITE_CANTOPEN;
const int _c_CSQLITE_PROTOCOL   = SQLITE_PROTOCOL;
const int _c_CSQLITE_EMPTY      = SQLITE_EMPTY;
const int _c_CSQLITE_SCHEMA     = SQLITE_SCHEMA;
const int _c_CSQLITE_TOOBIG     = SQLITE_TOOBIG;
const int _c_CSQLITE_CONSTRAINT = SQLITE_CONSTRAINT;
const int _c_CSQLITE_MISMATCH   = SQLITE_MISMATCH;
const int _c_CSQLITE_MISUSE     = SQLITE_MISUSE;
const int _c_CSQLITE_NOLFS      = SQLITE_NOLFS;
const int _c_CSQLITE_AUTH       = SQLITE_AUTH;
const int _c_CSQLITE_ROW        = SQLITE_ROW;
const int _c_CSQLITE_DONE       = SQLITE_DONE;

/* For sqlite3_column_type */
const int _c_CSQLITE_INTEGER  = SQLITE_INTEGER;
const int _c_CSQLITE_FLOAT    = SQLITE_FLOAT;
const int _c_CSQLITE_TEXT     = SQLITE_TEXT;
const int _c_CSQLITE_BLOB     = SQLITE_BLOB;
const int _c_CSQLITE_NULL     = SQLITE_NULL;

/* sqlite3_create_collation... */
const int _c_CSQLITE_UTF8     = SQLITE_UTF8;
const int _c_CSQLITE_UTF16LE  = SQLITE_UTF16LE;
const int _c_CSQLITE_UTF16BE  = SQLITE_UTF16BE;
const int _c_CSQLITE_UTF16    = SQLITE_UTF16;
const int _c_CSQLITE_ANY      = SQLITE_ANY;

/* for t_xAuth */
const int _c_CSQLITE_COPY = SQLITE_COPY;
const int _c_CSQLITE_CREATE_INDEX = SQLITE_CREATE_INDEX;
const int _c_CSQLITE_CREATE_TABLE = SQLITE_CREATE_TABLE;
const int _c_CSQLITE_CREATE_TEMP_INDEX = SQLITE_CREATE_TEMP_INDEX;
const int _c_CSQLITE_CREATE_TEMP_TABLE = SQLITE_CREATE_TEMP_TABLE;
const int _c_CSQLITE_CREATE_TEMP_TRIGGER = SQLITE_CREATE_TEMP_TRIGGER;
const int _c_CSQLITE_CREATE_TEMP_VIEW = SQLITE_CREATE_TEMP_VIEW;
const int _c_CSQLITE_CREATE_TRIGGER = SQLITE_CREATE_TRIGGER;
const int _c_CSQLITE_CREATE_VIEW = SQLITE_CREATE_VIEW;
const int _c_CSQLITE_DELETE = SQLITE_DELETE;
const int _c_CSQLITE_DROP_INDEX = SQLITE_DROP_INDEX;
const int _c_CSQLITE_DROP_TABLE = SQLITE_DROP_TABLE;
const int _c_CSQLITE_DROP_TEMP_INDEX = SQLITE_DROP_TEMP_INDEX;
const int _c_CSQLITE_DROP_TEMP_TABLE = SQLITE_DROP_TEMP_TABLE;
const int _c_CSQLITE_DROP_TEMP_TRIGGER = SQLITE_DROP_TEMP_TRIGGER;
const int _c_CSQLITE_DROP_TEMP_VIEW = SQLITE_DROP_TEMP_VIEW;
const int _c_CSQLITE_DROP_TRIGGER = SQLITE_DROP_TRIGGER;
const int _c_CSQLITE_DROP_VIEW = SQLITE_DROP_VIEW;
const int _c_CSQLITE_INSERT = SQLITE_INSERT;
const int _c_CSQLITE_PRAGMA = SQLITE_PRAGMA;
const int _c_CSQLITE_READ = SQLITE_READ;
const int _c_CSQLITE_SELECT = SQLITE_SELECT;
const int _c_CSQLITE_TRANSACTION = SQLITE_TRANSACTION;
const int _c_CSQLITE_UPDATE = SQLITE_UPDATE;
const int _c_CSQLITE_ATTACH = SQLITE_ATTACH;
const int _c_CSQLITE_DETACH = SQLITE_DETACH;
const int _c_CSQLITE_DENY = SQLITE_DENY;
const int _c_CSQLITE_IGNORE = SQLITE_IGNORE;

typedef void (*t_blob_destroy_proc) (void*);
typedef int (*t_busy_handler_proc) (void*, int);
typedef void (*t_collation_needed_proc) (void*, sqlite3*,int,const char*);
typedef void (*t_collation_needed16_proc) (void*,sqlite3*,int,const void*);
typedef void (*t_xFunc_proc) (sqlite3_context*,int,sqlite3_value**);
typedef void (*t_xStep_proc) (sqlite3_context*,int,sqlite3_value**);
typedef void (*t_xFinal_proc) (sqlite3_context*);
typedef int  (*t_xCompare)   (void*,int,const void*,int,const void*);
typedef int (*t_sqlite3_callback) (void*,int,char**, char**);
typedef void (*t_result_blob_proc) (void*);
typedef int (*t_xAuth) (void*,int,const char*,const char*,const char*,const char*);
typedef void (*t_xTrace) (void*,const char*);




void * _c_Csqlite3_aggregate_context (sqlite3_context* p, int nBytes)
{
  return sqlite3_aggregate_context (p, nBytes);
}

int _c_Csqlite3_aggregate_count (sqlite3_context* p)
{
  return sqlite3_aggregate_count (p);
}

int _c_Csqlite3_bind_blob (sqlite3_stmt* stmnt, int something, const void * p, int n, t_blob_destroy_proc f)
{
  return sqlite3_bind_blob (stmnt, something, p, n, f);
}

int _c_Csqlite3_bind_double (sqlite3_stmt* stmnt, int i, double d)
{
  return sqlite3_bind_double (stmnt, i, d);
}

int _c_Csqlite3_bind_int (sqlite3_stmt* stmnt, int i1, int i2)
{
  return sqlite3_bind_int (stmnt, i1, i2);
}

int _c_Csqlite3_bind_int64 (sqlite3_stmt* stmnt, int i, long long int lli)
{
  return sqlite3_bind_int64 (stmnt, i, lli);
}

int _c_Csqlite3_bind_null (sqlite3_stmt* stmnt, int i)
{
  return sqlite3_bind_null (stmnt, i);
}

int _c_Csqlite3_bind_parameter_count (sqlite3_stmt* stmnt)
{
  return sqlite3_bind_parameter_count (stmnt);
}

int _c_Csqlite3_bind_parameter_index (sqlite3_stmt* stmnt, const char *zName)
{
  return sqlite3_bind_parameter_index (stmnt, zName);
}

const char *_c_Csqlite3_bind_parameter_name (sqlite3_stmt* stmnt, int i)
{
  return sqlite3_bind_parameter_name (stmnt, i);
}

int _c_Csqlite3_bind_text (sqlite3_stmt* stmnt, int i, const char* txt_utf8, int n, t_blob_destroy_proc f)
{
  return sqlite3_bind_text (stmnt, i, txt_utf8, n, f);
}

int _c_Csqlite3_bind_text16 (sqlite3_stmt*  stmnt, int i, const void * txt_utf16, int n, t_blob_destroy_proc f)
{
  return sqlite3_bind_text16 (stmnt, i, txt_utf16, n, f);
}

int _c_Csqlite3_busy_handler (sqlite3* s, t_busy_handler_proc p, void * q)
{
  return sqlite3_busy_handler (s, p, q);
}

int _c_Csqlite3_busy_timeout (sqlite3* s, int ms)
{
  return sqlite3_busy_timeout (s, ms);
}

int _c_Csqlite3_changes (sqlite3* s)
{
  return sqlite3_changes (s);
}

/* FF: not in this version?
int _c_Csqlite3_clear_bindings (void * stmnt)
{
  return sqlite3_clear_bindings (stmnt);
}
**/

int _c_Csqlite3_close (sqlite3* s)
{
  return sqlite3_close (s);
}

int _c_Csqlite3_collation_needed (sqlite3* s, void * p, t_collation_needed_proc f)
{
  return sqlite3_collation_needed (s, p, f);
}

int _c_Csqlite3_collation_needed16 (sqlite3*  s, void * p, t_collation_needed16_proc f)
{
  return sqlite3_collation_needed16 (s, p, f);
}

const void *_c_Csqlite3_column_blob (sqlite3_stmt* stmnt, int iCol)
{
  return sqlite3_column_blob (stmnt, iCol);
}

int _c_Csqlite3_column_bytes (sqlite3_stmt* stmnt, int iCol)
{
  return sqlite3_column_bytes (stmnt, iCol);
}

int _c_Csqlite3_column_bytes16 (sqlite3_stmt* stmnt, int iCol)
{
  return sqlite3_column_bytes16 (stmnt, iCol);
}

int _c_Csqlite3_column_count (sqlite3_stmt *pStmt)
{
  return sqlite3_column_count (pStmt);
}

const char *_c_Csqlite3_column_decltype (sqlite3_stmt *stmnt, int i)
{
  return sqlite3_column_decltype (stmnt, i);
}

const void *_c_Csqlite3_column_decltype16 (sqlite3_stmt* stmnt,int i)
{
  return sqlite3_column_decltype16 (stmnt, i);
}

double _c_Csqlite3_column_double (sqlite3_stmt* stmnt, int iCol)
{
  return sqlite3_column_double (stmnt, iCol);
}

int _c_Csqlite3_column_int (sqlite3_stmt* stmnt, int iCol)
{
  return sqlite3_column_int (stmnt, iCol);
}

long long int _c_Csqlite3_column_int64 (sqlite3_stmt* stmnt, int iCol)
{
  return sqlite3_column_int64 (stmnt, iCol);
}

const char * _c_Csqlite3_column_name (sqlite3_stmt* stmnt, int i)
{
  return sqlite3_column_name (stmnt, i);
}

const void * _c_Csqlite3_column_name16 (sqlite3_stmt* stmnt, int i)
{
  return sqlite3_column_name16 (stmnt, i);
}

const unsigned char *_c_Csqlite3_column_text (sqlite3_stmt* stmnt, int iCol)
{
  return sqlite3_column_text (stmnt, iCol);
}

const void*  _c_Csqlite3_column_text16 (sqlite3_stmt* stmnt, int iCol)
{
  return sqlite3_column_text16 (stmnt, iCol);
}

int _c_Csqlite3_column_type (sqlite3_stmt* stmnt, int iCol)
{
  return sqlite3_column_type (stmnt, iCol);
}

int _c_Csqlite3_complete (const char *sql)
{
  return sqlite3_complete (sql) ? 1 : 0;
}

int _c_Csqlite3_complete16 (const void * sql)
{
  return sqlite3_complete16 (sql) ? 1 : 0;
}

int _c_Csqlite3_create_collation (sqlite3* sql, const char *zName, int eTextRep, void * p1, t_xCompare f)
{
  return sqlite3_create_collation (sql, zName, eTextRep, p1, f);
}

int _c_Csqlite3_create_collation16 (sqlite3* sql, const char *zName, int eTextRep, void * p, t_xCompare f)
{
  return sqlite3_create_collation16 (sql, zName, eTextRep, p, f);
}
                    
int _c_Csqlite3_create_function (sqlite3* p1, const char *zFunctionName,int nArg,int eTextRep, void * pUserData, t_xFunc_proc f1, t_xStep_proc f2, t_xFinal_proc f3)
{
  return sqlite3_create_function (p1, zFunctionName, nArg, eTextRep, pUserData, f1, f2, f3);
}

int _c_Csqlite3_create_function16 (sqlite3* p1 ,const void *zFunctionName,int nArg,int eTextRep, void * pUserData, t_xFunc_proc f1, t_xStep_proc f2, t_xFinal_proc f3)
{
  return sqlite3_create_function16 (p1, zFunctionName, nArg, eTextRep, pUserData, f1, f2, f3);
}

int _c_Csqlite3_data_count (sqlite3_stmt* stmnt)
{
  return sqlite3_data_count (stmnt);
}

/* FF: not in this version */
/** int _c_Csqlite3_db_handle (void * stmnt)
{
  return sqlite3_db_handle (stmnt);
}
**/

int _c_Csqlite3_errcode (sqlite3* db)
{
  return sqlite3_errcode (db);
}

const char *_c_Csqlite3_errmsg (sqlite3* db)
{
  return sqlite3_errmsg (db);
}

const void *_c_Csqlite3_errmsg16 (sqlite3* db)
{
  return sqlite3_errmsg16 (db);
}

int _c_Csqlite3_exec (sqlite3* db, const char *sql, t_sqlite3_callback f, void * p, char **errmsg)
{
  return sqlite3_exec (db, sql, f, p, errmsg);
}

/* FF nt in this version 
int _c_Csqlite3_expired (sqlite3_stmt* stmnt)
{
  return sqlite3_expired (stmnt);
}
*/

int _c_Csqlite3_finalize (sqlite3_stmt *pStmt)
{
  return sqlite3_finalize (pStmt);
}

void _c_Csqlite3_free (char *z)
{
  sqlite3_free (z);
}

void _c_Csqlite3_free_table (char **result)
{
  sqlite3_free_table (result);
}

/* Result written to a char *[]  that this points to */
int _c_Csqlite3_get_table (sqlite3* db,const char *sql,char ***resultp,int *nrow,int *ncolumn,char **errmsg)
{
  return sqlite3_get_table (db, sql, resultp, nrow, ncolumn, errmsg);
}

/** FF not in this version 
int _c_Csqlite3_get_autocommit (sqlite3*db)
{
  return sqlite3_get_autocommit (db);
}
**/

/* FF not in this version
int _c_Csqlite3_global_recover ()
{
  return sqlite3_global_recover ();
}
*/

void _c_Csqlite3_interrupt (sqlite3*db)
{
  sqlite3_interrupt (db);
}

/* FF ??
int _c_Csqlite3_key (sqlite3 *db, const void *pKey, int nKey)
{
  return sqlite3_key (db, pKey, nKey);
}
*/

long long int _c_Csqlite3_last_insert_rowid (sqlite3* db)
{
  return sqlite3_last_insert_rowid (db);
}

const char *_c_Csqlite3_libversion (void)
{
  return sqlite3_libversion ();
}

int _c_Csqlite3_open (const char *filename, sqlite3 **ppDb)
{
  return sqlite3_open (filename, ppDb);
}

int _c_Csqlite3_open16 (const void * filename,sqlite3 **ppDb)
{
  return sqlite3_open16 (filename, ppDb);
}

int _c_Csqlite3_prepare (sqlite3 *db,const char *zSql,int nBytes,sqlite3_stmt **ppStmt,const char **pzTail)
{
  return sqlite3_prepare (db, zSql, nBytes, ppStmt, pzTail);
}

int _c_Csqlite3_prepare16 (sqlite3 *db,const void *zSql,int nBytes,sqlite3_stmt **ppStmt,const void **pzTail)
{
  return sqlite3_prepare16 (db, zSql, nBytes, ppStmt, pzTail);
}

/* FF ???
int _c_Csqlite3_rekey (sqlite3 *db, const void *pKey, int nKey)
{
  return sqlite3_rekey (db, pKey, nKey);
}    
*/

int _c_Csqlite3_reset (sqlite3_stmt *pStmt)
{
  return sqlite3_reset (pStmt);
}

void _c_Csqlite3_result_blob (sqlite3_context*ctx, const void * p, int n, t_result_blob_proc f)
{
  sqlite3_result_blob (ctx, p, n, f);
}

void _c_Csqlite3_result_double (sqlite3_context*ctx, double d)
{
  sqlite3_result_double (ctx, d);
}

void _c_Csqlite3_result_error (sqlite3_context*ctx, const char*s, int n)
{
  sqlite3_result_error (ctx, s, n);
}

void _c_Csqlite3_result_error16 (sqlite3_context*ctx, const void * p, int n)
{
  sqlite3_result_error16 (ctx, p, n);
}

void _c_Csqlite3_result_int (sqlite3_context*ctx, int n)
{
  sqlite3_result_int (ctx, n);
}

void _c_Csqlite3_result_int64 (sqlite3_context*ctx, long long int r)
{
  sqlite3_result_int64 (ctx, r);
}

void _c_Csqlite3_result_null (sqlite3_context*ctx)
{
  sqlite3_result_null (ctx);
}

void _c_Csqlite3_result_text (sqlite3_context*ctx, const char* s, int n, t_result_blob_proc f)
{
  sqlite3_result_text (ctx, s, n, f);
}

void _c_Csqlite3_result_text16 (sqlite3_context*ctx, const void * p, int n, t_result_blob_proc f)
{
  sqlite3_result_text16 (ctx, p, n, f);
}

void _c_Csqlite3_result_text16be (sqlite3_context*ctx, const void * p, int n, t_result_blob_proc f)
{
  sqlite3_result_text16be (ctx, p, n, f);
}

void _c_Csqlite3_result_text16le (sqlite3_context*ctx, const void * p, int n, t_result_blob_proc f)
{
  sqlite3_result_text16le (ctx, p, n, f);
}

void _c_Csqlite3_result_value (sqlite3_context*ctx, sqlite3_value*v)
{
  sqlite3_result_value (ctx, v);
}

int _c_Csqlite3_set_authorizer (sqlite3* sql, t_xAuth f, void * pUserData)
{
  return sqlite3_set_authorizer (sql, f, pUserData);
}

/** FF not in this version
int _c_Csqlite3_sleep (int ms)
{
  return sqlite3_sleep (ms);
}
**/

int _c_Csqlite3_step (sqlite3_stmt* stmnt)
{
  return sqlite3_step (stmnt);
}

int _c_Csqlite3_total_changes (sqlite3* sql)
{
  return sqlite3_total_changes (sql);
}

void * _c_Csqlite3_trace (sqlite3* sql, t_xTrace f, void * a)
{
  return sqlite3_trace (sql, f, a);
}

/** FF not in this version
int _c_Csqlite3_transfer_bindings (sqlite3_stmt* stmnt1, sqlite3_stmt* stmnt2)
{
  return sqlite3_transfer_bindings (stmnt1, stmnt2);
}
**/

void * _c_Csqlite3_user_data (sqlite3_context* ctx)
{
  return sqlite3_user_data (ctx);
}

const void * _c_Csqlite3_value_blob (sqlite3_value* val)
{
  return sqlite3_value_blob (val);
}

int _c_Csqlite3_value_bytes (sqlite3_value* val)
{
  return sqlite3_value_bytes (val);
}

int _c_Csqlite3_value_bytes16 (sqlite3_value* val)
{
  return sqlite3_value_bytes16 (val);
}

double _c_Csqlite3_value_double (sqlite3_value* val)
{
  return sqlite3_value_double (val);
}

int _c_Csqlite3_value_int (sqlite3_value* val)
{
  return sqlite3_value_int (val);
}

long long int _c_Csqlite3_value_int64 (sqlite3_value* val)
{
  return sqlite3_value_int64 (val);
}

const unsigned char * _c_Csqlite3_value_text (sqlite3_value* val)
{
  return sqlite3_value_text (val);
}

const void * _c_Csqlite3_value_text16 (sqlite3_value* val)
{
  return sqlite3_value_text16 (val);
}

const void * _c_Csqlite3_value_text16be (sqlite3_value* val)
{
  return sqlite3_value_text16be (val);
}

const void * _c_Csqlite3_value_text16le (sqlite3_value* val)
{
  return sqlite3_value_text16le (val);
}

int _c_Csqlite3_value_type (sqlite3_value* val)
{
  return sqlite3_value_type (val);
}
